"""
    Video categories resource implementation.
"""

from typing import Optional, Union

from pyyoutube.error import PyYouTubeException, ErrorMessage, ErrorCode
from pyyoutube.resources.base_resource import Resource
from pyyoutube.models import VideoCategoryListResponse
from pyyoutube.utils.params_checker import enf_comma_separated, enf_parts


class VideoCategoriesResource(Resource):
    """A videoCategory resource identifies a category that has been or could be associated with uploaded videos.

    References: https://developers.google.com/youtube/v3/docs/videoCategories
    """

    def list(
        self,
        parts: Optional[Union[str, list, tuple, set]] = None,
        category_id: Optional[Union[str, list, tuple, set]] = None,
        region_code: Optional[str] = None,
        hl: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, VideoCategoryListResponse]:
        """Returns a list of categories that can be associated with YouTube videos.

        Args:
            parts:
                Comma-separated list of one or more video category resource properties.
                Accepted values: snippet
            category_id:
                Specifies a comma-separated list of video category IDs for the resources that you are retrieving.
            region_code:
                Instructs the API to return the list of video categories available in the specified country.
                The parameter value is an ISO 3166-1 alpha-2 country code.
            hl:
                Specifies the language that should be used for text values in the API response.
                The default value is en_US.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Video category data.
        Raises:
            PyYouTubeException: Missing filter parameter.
        """
        params = {
            "part": enf_parts(resource="videoCategories", value=parts),
            "hl": hl,
            **kwargs,
        }

        if category_id is not None:
            params["id"] = enf_comma_separated(field="category_id", value=category_id)
        elif region_code is not None:
            params["regionCode"] = region_code
        else:
            raise PyYouTubeException(
                ErrorMessage(
                    status_code=ErrorCode.MISSING_PARAMS,
                    message=f"Specify at least one of category_id or region_code",
                )
            )
        response = self._client.request(path="videoCategories", params=params)
        data = self._client.parse_response(response=response)
        return data if return_json else VideoCategoryListResponse.from_dict(data)



from enum import Enum


class LanguageCode(Enum):
    EN = 'en'
    KO = 'ko'
    # 여기에 추가 언어 코드를 정의할 수 있습니다.


class RegionCode(Enum):
    US = 'US'
    KR = 'KR'
    # 여기에 추가 국가 코드를 정의할 수 있습니다.


class DataParameterBuilder:
    """
    YouTube API 요청에 필요한 데이터 매개변수를 구성하기 위한 빌더 클래스.
    이 클래스를 사용하여 유연하게 매개변수를 설정 가능.
    """

    def __init__(self):
        # 기본 매개변수 설정
        self._params = {
            'context': {
                'client': {
                    'hl': 'ko',  # 기본 언어 설정 (en 영어)
                    'gl': 'KR',  # 기본 지역 설정 (US 미국)
                    'clientName': 'WEB',
                    'clientVersion': '2.20210804.02.00',
                },
            },
        }

    def set_language(self, language: LanguageCode):
        """
        사용할 언어를 설정.
        :param language: 설정할 언어 코드 (예: 'ko', 'en')
        :return: 빌더 객체 자신을 반환 (메서드 체이닝을 위함).
        """
        self._params['context']['client']['hl'] = language.value
        return self

    def set_region(self, region: RegionCode):
        """
        사용할 지역을 설정.
        :param region: 설정할 지역 코드 (예: 'KR', 'US')
        :return: 빌더 객체 자신을 반환.
        """
        self._params['context']['client']['gl'] = region.value
        return self

    def set_continuation(self, continuation):
        """
        Continuation 토큰을 설정.
        :param continuation: 설정할 continuation 토큰 값.
        :return: 빌더 객체 자신을 반환.
        """
        self._params['continuation'] = continuation.replace('=', '%3D')
        return self

    def set_param(self, param):
        """
        파라미터 값을 설정.
        :param param: 설정할 파라미터 값.
        :return: 빌더 객체 자신을 반환.
        """
        self._params['params'] = param.replace('=', '%3D')
        return self

    def build(self):
        """
        설정된 매개변수를 바탕으로 최종 데이터 매개변수 객체를 생성.
        :return: 생성된 데이터 매개변수 객체.
        """
        return self._params



import logging
from string import Template

# 현재 모듈에 대한 로거를 설정합니다.
logger = logging.getLogger(__name__)


class EmbeddingIFrameAPICreator:
    # Embedding을 위한 iframe API HTML의 템플릿을 정의.
    IFRAME_HTML_TEMPLATE = """
            <iframe width="560" height="315" src="https://www.youtube.com/embed/{}" title="YouTube video 
            player" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; 
            picture-in-picture; web-share" allowfullscreen></iframe> """

    def __init__(self):
        # YouTube의 기본 임베딩 URL을 설정합니다.
        self.base_url = 'https://www.youtube.com/embed'

    @classmethod
    def create(cls, video_id: str):
        """임베딩한 URL을 생성하고 반환"""
        # video_id가 문자열이 아닌 경우 ValueError를 발생
        if not isinstance(video_id, str):
            raise ValueError("Parameter video_id must be str type")
        # string.Template을 사용하여 iframe HTML 템플릿을 생성
        template = Template(cls.IFRAME_HTML_TEMPLATE)
        # video_id를 사용하여 IFrame HTML을 생성
        iframe_html = template.substitute(video_id=video_id)
        return iframe_html



"""
    These are caption related models
"""

from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel
from .mixins import DatetimeTimeMixin
from .common import BaseResource, BaseApiResponse


@dataclass
class CaptionSnippet(BaseModel, DatetimeTimeMixin):
    """
    A class representing the caption snippet resource info.

    Refer: https://developers.google.com/youtube/v3/docs/captions#snippet
    """

    videoId: Optional[str] = field(default=None)
    lastUpdated: Optional[str] = field(default=None)
    trackKind: Optional[str] = field(default=None, repr=False)
    language: Optional[str] = field(default=None, repr=False)
    name: Optional[str] = field(default=None, repr=False)
    audioTrackType: Optional[str] = field(default=None, repr=False)
    isCC: Optional[bool] = field(default=None, repr=False)
    isLarge: Optional[bool] = field(default=None, repr=False)
    isEasyReader: Optional[bool] = field(default=None, repr=False)
    isDraft: Optional[bool] = field(default=None, repr=False)
    isAutoSynced: Optional[bool] = field(default=None, repr=False)
    status: Optional[str] = field(default=None, repr=False)
    failureReason: Optional[str] = field(default=None, repr=False)


@dataclass
class Caption(BaseResource):
    """
    A class representing the caption resource info.

    Refer: https://developers.google.com/youtube/v3/docs/captions
    """

    snippet: Optional[CaptionSnippet] = field(default=None)


@dataclass
class CaptionListResponse(BaseApiResponse):
    """
    A class representing the activity response info.

    Refer: https://developers.google.com/youtube/v3/docs/captions/list?#response_1
    """

    items: Optional[List[Caption]] = field(default=None, repr=False)



"""
    Those are models related to channel sections.
"""

from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel
from .common import BaseResource, BaseApiResponse


@dataclass
class ChannelSectionSnippet(BaseModel):
    """
    A class representing the channel section snippet info.

    Refer: https://developers.google.com/youtube/v3/docs/channelSections#snippet
    """

    type: Optional[str] = field(default=None)
    channelId: Optional[str] = field(default=None, repr=False)
    title: Optional[str] = field(default=None, repr=False)
    position: Optional[int] = field(default=None)


@dataclass
class ChannelSectionContentDetails(BaseModel):
    """
    A class representing the channel section content details info.

    Refer: https://developers.google.com/youtube/v3/docs/channelSections#contentDetails
    """

    playlists: Optional[List[str]] = field(default=None, repr=False)
    channels: Optional[List[str]] = field(default=None)


@dataclass
class ChannelSection(BaseResource):
    """
    A class representing the channel section info.

    Refer: https://developers.google.com/youtube/v3/docs/channelSections#properties
    """

    snippet: Optional[ChannelSectionSnippet] = field(default=None, repr=False)
    contentDetails: Optional[ChannelSectionContentDetails] = field(
        default=None, repr=False
    )


@dataclass
class ChannelSectionResponse(BaseApiResponse):
    """
    A class representing the channel section's retrieve response info.

    Refer: https://developers.google.com/youtube/v3/docs/channelSections/list?#properties_1
    """

    items: Optional[List[ChannelSection]] = field(default=None, repr=False)


@dataclass
class ChannelSectionListResponse(ChannelSectionResponse):
    ...



"""
    These are comment related models.
"""

from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel
from .mixins import DatetimeTimeMixin
from .common import BaseApiResponse, BaseResource


@dataclass
class CommentSnippetAuthorChannelId(BaseModel):
    """
    A class representing comment's snippet authorChannelId info.

    Refer: https://developers.google.com/youtube/v3/docs/comments#snippet.authorChannelId
    """

    value: Optional[str] = field(default=None)


@dataclass
class CommentSnippet(BaseModel, DatetimeTimeMixin):
    """
    A class representing comment's snippet info.

    Refer: https://developers.google.com/youtube/v3/docs/comments#snippet
    """

    authorDisplayName: Optional[str] = field(default=None)
    authorProfileImageUrl: Optional[str] = field(default=None, repr=False)
    authorChannelUrl: Optional[str] = field(default=None, repr=False)
    authorChannelId: Optional[CommentSnippetAuthorChannelId] = field(
        default=None, repr=False
    )
    channelId: Optional[str] = field(default=None, repr=False)
    # videoId has deprecated, see https://developers.google.com/youtube/v3/revision_history#november-09,-2023
    videoId: Optional[str] = field(default=None, repr=False)
    textDisplay: Optional[str] = field(default=None, repr=False)
    textOriginal: Optional[str] = field(default=None, repr=False)
    parentId: Optional[str] = field(default=None, repr=False)
    canRate: Optional[bool] = field(default=None, repr=False)
    viewerRating: Optional[str] = field(default=None, repr=False)
    likeCount: Optional[int] = field(default=None)
    moderationStatus: Optional[str] = field(default=None, repr=False)
    publishedAt: Optional[str] = field(default=None, repr=False)
    updatedAt: Optional[str] = field(default=None, repr=False)


@dataclass
class Comment(BaseResource):
    """
    A class representing comment info.

    Refer: https://developers.google.com/youtube/v3/docs/comments
    """

    snippet: Optional[CommentSnippet] = field(default=None)


@dataclass
class CommentListResponse(BaseApiResponse):
    """
    A class representing the comment's retrieve response info.

    Refer: https://developers.google.com/youtube/v3/docs/comments/list#response_1
    """

    items: Optional[List[Comment]] = field(default=None, repr=False)



"""
    Members resource implementation.
"""

from typing import Optional, Union

from pyyoutube.resources.base_resource import Resource
from pyyoutube.models import MemberListResponse
from pyyoutube.utils.params_checker import enf_parts, enf_comma_separated


class MembersResource(Resource):
    """A member resource represents a channel member for a YouTube channel.

    References: https://developers.google.com/youtube/v3/docs/members
    """

    def list(
        self,
        parts: Optional[Union[str, list, tuple, set]] = None,
        mode: Optional[str] = None,
        max_results: Optional[int] = None,
        page_token: Optional[str] = None,
        has_access_to_level: Optional[str] = None,
        filter_by_member_channel_id: Optional[Union[str, list, tuple, set]] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, MemberListResponse]:
        """Lists members (formerly known as "sponsors") for a channel.

        Args:
            parts:
                Comma-separated list of one or more channel resource properties.
                Accepted values: snippet
            mode:
                Indicates which members will be included in the API response.
                Accepted values:
                    - all_current: List current members, from newest to oldest.
                    - updates: List only members that joined or upgraded since the previous API call.
            max_results:
                The parameter specifies the maximum number of items that should be returned
                the result set.
                Acceptable values are 0 to 1000, inclusive. The default value is 5.
            page_token:
                The parameter identifies a specific page in the result set that should be returned.
            has_access_to_level:
                A level ID that specifies the minimum level that members in the result set should have.
            filter_by_member_channel_id:
                specifies a comma-separated list of channel IDs that can be used to check the membership
                status of specific users.
                Maximum of 100 channels can be specified per call.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Members data.
        """

        params = {
            "part": enf_parts(resource="members", value=parts),
            "mode": mode,
            "maxResults": max_results,
            "pageToken": page_token,
            "hasAccessToLevel": has_access_to_level,
            "filterByMemberChannelId": enf_comma_separated(
                field="filter_by_member_channel_id", value=filter_by_member_channel_id
            ),
            **kwargs,
        }
        response = self._client.request(path="members", params=params)
        data = self._client.parse_response(response=response)
        return data if return_json else MemberListResponse.from_dict(data)



"""
    Channel Section resource implementation.
"""

from typing import Optional, Union

from pyyoutube.error import PyYouTubeException, ErrorMessage, ErrorCode
from pyyoutube.resources.base_resource import Resource
from pyyoutube.models import ChannelSection, ChannelSectionListResponse
from pyyoutube.utils.params_checker import enf_comma_separated, enf_parts


class ChannelSectionsResource(Resource):
    """A channelSection resource contains information about a set of videos that a channel has chosen to feature.

    References: https://developers.google.com/youtube/v3/docs/channelSections
    """

    def list(
        self,
        parts: Optional[Union[str, list, tuple, set]] = None,
        channel_id: Optional[str] = None,
        section_id: Optional[Union[str, list, tuple, set]] = None,
        mine: Optional[bool] = None,
        hl: Optional[str] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, ChannelSectionListResponse]:
        """Returns a list of channelSection resources that match the API request criteria.

        Args:
            parts:
                Comma-separated list of one or more channel resource properties.
            channel_id:
                ID for the channel which you want to retrieve sections.
            section_id:
                Specifies a comma-separated list of IDs that uniquely identify the channelSection
                resources that are being retrieved.
            mine:
                Set this parameter's value to true to retrieve a feed of the channel sections
                associated with the authenticated user's YouTube channel.
            hl:
                The hl parameter provided support for retrieving localized metadata for a channel section.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Channel section data.

        Raises:
            PyYouTubeException: Missing filter parameter.

        """
        params = {
            "part": enf_parts(resource="channelSections", value=parts),
            "hl": hl,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        if channel_id is not None:
            params["channelId"] = channel_id
        elif section_id is not None:
            params["id"] = enf_comma_separated(field="section_id", value=section_id)
        elif mine is not None:
            params["mine"] = mine
        else:
            raise PyYouTubeException(
                ErrorMessage(
                    status_code=ErrorCode.MISSING_PARAMS,
                    message=f"Specify at least one of for_username, id, managedByMe or mine",
                )
            )
        response = self._client.request(path="channelSections", params=params)
        data = self._client.parse_response(response=response)
        return data if return_json else ChannelSectionListResponse.from_dict(data)

    def insert(
        self,
        body: Union[dict, ChannelSection],
        parts: Optional[Union[str, list, tuple, set]] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        on_behalf_of_content_owner_channel: Optional[str] = None,
        return_json: bool = False,
        **kwargs,
    ) -> Union[dict, ChannelSection]:
        """Adds a channel section to the authenticated user's channel.
        A channel can create a maximum of 10 shelves.

        Args:
            parts:
                The part parameter serves two purposes in this operation. It identifies the properties
                that the write operation will set as well as the properties that the API response will include.
                Accept values:
                    - id
                    - contentDetails
                    - snippet
            body:
                Provide a channelSection resource in the request body. You can give dataclass or just a dict with data.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            on_behalf_of_content_owner_channel:
                The onBehalfOfContentOwnerChannel parameter specifies the YouTube channel ID of the
                channel to which a video is being added. This parameter is required when a request
                specifies a value for the onBehalfOfContentOwner parameter, and it can only be used
                in conjunction with that parameter. In addition, the request must be authorized
                using a CMS account that is linked to the content owner that the onBehalfOfContentOwner
                parameter specifies. Finally, the channel that the onBehalfOfContentOwnerChannel parameter
                value specifies must be linked to the content owner that the onBehalfOfContentOwner parameter specifies.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Channel section data.
        """
        params = {
            "part": enf_parts(resource="channelSections", value=parts),
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            "onBehalfOfContentOwnerChannel": on_behalf_of_content_owner_channel,
            **kwargs,
        }
        response = self._client.request(
            method="POST",
            path="channelSections",
            params=params,
            json=body,
        )
        data = self._client.parse_response(response=response)
        return data if return_json else ChannelSection.from_dict(data)

    def update(
        self,
        body: Union[dict, ChannelSection],
        parts: Optional[Union[str, list, tuple, set]] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        return_json: bool = False,
        **kwargs,
    ) -> Union[dict, ChannelSection]:
        """Updates a channel section.

        Args:
            parts:
                The part parameter serves two purposes in this operation. It identifies the properties
                that the write operation will set as well as the properties that the API response will include.
                Accept values:
                    - id
                    - contentDetails
                    - snippet
            body:
                Provide a channelSection resource in the request body. You can give dataclass or just a dict with data.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Channel section data.
        """
        params = {
            "part": enf_parts(resource="channelSections", value=parts),
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="PUT",
            path="channelSections",
            params=params,
            json=body,
        )
        data = self._client.parse_response(response=response)
        return data if return_json else ChannelSection.from_dict(data)

    def delete(
        self,
        section_id: str,
        on_behalf_of_content_owner: Optional[str] = None,
        **kwargs,
    ) -> bool:
        """Deletes a channel section.

        Args:
            section_id:
                ID for the target channel section.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Channel section delete status
        """
        params = {
            "id": section_id,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="DELETE",
            path="channelSections",
            params=params,
        )
        if response.ok:
            return True
        self._client.parse_response(response=response)



"""
    Videos resource implementation.
"""

from typing import Optional, Union

from pyyoutube.error import PyYouTubeException, ErrorCode, ErrorMessage
from pyyoutube.resources.base_resource import Resource
from pyyoutube.media import Media, MediaUpload
from pyyoutube.models import (
    Video,
    VideoListResponse,
    VideoGetRatingResponse,
    VideoReportAbuse,
)
from pyyoutube.utils.params_checker import enf_comma_separated, enf_parts


class VideosResource(Resource):
    """A video resource represents a YouTube video.

    References: https://developers.google.com/youtube/v3/docs/videos
    """

    def list(
        self,
        parts: Optional[Union[str, list, tuple, set]] = None,
        chart: Optional[str] = None,
        video_id: Optional[Union[str, list, tuple, set]] = None,
        my_rating: Optional[str] = None,
        hl: Optional[str] = None,
        max_height: Optional[int] = None,
        max_results: Optional[int] = None,
        max_width: Optional[int] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        page_token: Optional[str] = None,
        region_code: Optional[str] = None,
        video_category_id: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, VideoListResponse]:
        """Returns a list of videos that match the API request parameters.

        Args:
            parts:
                Comma-separated list of one or more channel resource properties.
                Accepted values: id,contentDetails,fileDetails,liveStreamingDetails,
                localizations,player,processingDetails,recordingDetails,snippet,statistics,
                status,suggestions,topicDetails
            chart:
                Identifies the chart that you want to retrieve.
                Acceptable values are:
                    - mostPopular:  Return the most popular videos for the specified content region and video category.
            video_id:
                Specifies a comma-separated list of the YouTube video ID(s) for the resource(s) that are being retrieved.
            my_rating:
                Set this parameter's value to like or dislike to instruct the API to only return videos liked
                or disliked by the authenticated user.
                Acceptable values are:
                    - dislike: Returns only videos disliked by the authenticated user.
                    - like: Returns only video liked by the authenticated user.
            hl:
                Instructs the API to retrieve localized resource metadata for a specific application language
                that the YouTube website supports.
            max_height:
                Specifies the maximum height of the embedded player returned the player.embedHtml property.
            max_results:
                Specifies the maximum number of items that should be returned the result set.
            max_width:
                Specifies the maximum width of the embedded player returned the player.embedHtml property.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many difference YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            page_token:
                The parameter identifies a specific page in the result set that should be returned.
            region_code:
                Instructs the API to select a video chart available in the specified region.
            video_category_id:
                Identifies the video category for which the chart should be retrieved.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Videos data.
        Raises:
            PyYouTubeException: Missing filter parameter.
        """
        params = {
            "part": enf_parts(resource="videos", value=parts),
            "hl": hl,
            "maxHeight": max_height,
            "maxResults": max_results,
            "maxWidth": max_width,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            "pageToken": page_token,
            "regionCode": region_code,
            "videoCategoryId": video_category_id,
            **kwargs,
        }
        if chart is not None:
            params["chart"] = chart
        elif video_id is not None:
            params["id"] = enf_comma_separated(field="video_id", value=video_id)
        elif my_rating is not None:
            params["myRating"] = my_rating
        else:
            raise PyYouTubeException(
                ErrorMessage(
                    status_code=ErrorCode.MISSING_PARAMS,
                    message=f"Specify at least one of for_username,channel_id,managedByMe or mine",
                )
            )
        response = self._client.request(path="videos", params=params)
        data = self._client.parse_response(response=response)
        return data if return_json else VideoListResponse.from_dict(data)

    def insert(
        self,
        body: Union[dict, Video],
        media: Media,
        parts: Optional[Union[str, list, tuple, set]] = None,
        notify_subscribers: Optional[bool] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        on_behalf_of_content_owner_channel: Optional[str] = None,
        **kwargs,
    ) -> MediaUpload:
        """Uploads a video to YouTube and optionally sets the video's metadata.

        Example:

            import pyyoutube.models as mds
            from pyyoutube.media import Media

            body = mds.Video(
                snippet=mds.VideoSnippet(
                    title="video title",
                    description="video description"
                )
            )

            media = Media(filename="video.mp4")

            upload = client.videos.insert(
                body=body,
                media=media,
                parts=["snippet"],
            )

            response = None
            while response is None:
                status, response = upload.next_chunk()
                if status:
                    print(f"Upload {int(status.progress() * 100)} complete.")

            print(f"Response body: {response}")

        Args:
            body:
                Provide video data in the request body. You can give dataclass or just a dict with data.
            media:
                Media data to upload.
            parts:
                Comma-separated list of one or more channel resource properties.
                Accepted values: id,contentDetails,fileDetails,liveStreamingDetails,
                localizations,player,processingDetails,recordingDetails,snippet,statistics,
                status,suggestions,topicDetails
            notify_subscribers:
                Indicates whether YouTube should send a notification about the new video to users who
                subscribe to the video's channel
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            on_behalf_of_content_owner_channel:
                The onBehalfOfContentOwnerChannel parameter specifies the YouTube channel ID of the channel
                to which a video is being added. This parameter is required when a request specifies a value
                for the onBehalfOfContentOwner parameter, and it can only be used in conjunction with that
                parameter. In addition, the request must be authorized using a CMS account that is linked to
                the content owner that the onBehalfOfContentOwner parameter specifies. Finally, the channel
                that the onBehalfOfContentOwnerChannel parameter value specifies must be linked to the content
                owner that the onBehalfOfContentOwner parameter specifies.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.
        Returns:
            Video data.

        """
        params = {
            "part": enf_parts(resource="videos", value=parts),
            "notifySubscribers": notify_subscribers,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            "onBehalfOfContentOwnerChannel": on_behalf_of_content_owner_channel,
            **kwargs,
        }

        # Build a media upload instance.
        media_upload = MediaUpload(
            client=self._client,
            resource="videos",
            media=media,
            params=params,
            body=body.to_dict_ignore_none(),
        )
        return media_upload

    def update(
        self,
        body: Union[dict, Video],
        parts: Optional[Union[str, list, tuple, set]] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, Video]:
        """Updates a video's metadata.

        Args:
            body:
                Provide video data in the request body. You can give dataclass or just a dict with data.
            parts:
                Comma-separated list of one or more channel resource properties.
                Accepted values: id,contentDetails,fileDetails,liveStreamingDetails,
                localizations,player,processingDetails,recordingDetails,snippet,statistics,
                status,suggestions,topicDetails
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many difference YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Video updated data.
        """
        params = {
            "part": enf_parts(resource="videos", value=parts),
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="PUT",
            path="videos",
            params=params,
            json=body,
        )
        data = self._client.parse_response(response=response)
        return data if return_json else Video.from_dict(data)

    def rate(
        self,
        video_id: str,
        rating: Optional[str] = None,
        **kwargs: Optional[dict],
    ) -> bool:
        """Add a like or dislike rating to a video or remove a rating from a video.

        Args:
            video_id:
                Specifies the YouTube video ID of the video that is being rated or having its rating removed.
            rating:
                Specifies the rating to record.
                Acceptable values are:
                    - dislike: Records that the authenticated user disliked the video.
                    - like: Records that the authenticated user liked the video.
                    - none: Removes any rating that the authenticated user had previously set for the video.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Video rating status
        """
        params = {
            "id": video_id,
            "rating": rating,
            **kwargs,
        }
        response = self._client.request(
            method="POST",
            path="videos/rate",
            params=params,
        )
        if response.ok:
            return True
        self._client.parse_response(response=response)

    def get_rating(
        self,
        video_id: Union[str, list, tuple, set],
        on_behalf_of_content_owner: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, VideoGetRatingResponse]:
        """Retrieves the ratings that the authorized user gave to a list of specified videos.

        Args:
            video_id:
                Specifies a comma-separated list of the YouTube video ID(s).
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Video rating data.
        """

        params = {
            "id": enf_comma_separated(field="video_id", value=video_id),
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(path="videos/getRating", params=params)
        data = self._client.parse_response(response=response)
        return data if return_json else VideoGetRatingResponse.from_dict(data)

    def report_abuse(
        self,
        body: Optional[Union[dict, VideoReportAbuse]],
        on_behalf_of_content_owner: Optional[str] = None,
        **kwargs: Optional[dict],
    ) -> bool:
        """Reports a video for containing abusive content.

        Args:
            body:
                Provide report abuse data in the request body. You can give dataclass or just a dict with data.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            report status.
        """
        params = {
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="POST",
            path="videos/reportAbuse",
            params=params,
            json=body,
        )
        if response.ok:
            return True
        self._client.parse_response(response=response)

    def delete(
        self,
        video_id: str,
        on_behalf_of_content_owner: Optional[str] = None,
        **kwargs: Optional[dict],
    ) -> bool:
        """Deletes a YouTube video.

        Args:
            video_id:
                Specifies the YouTube video ID for the resource that is being deleted.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            video delete status.
        """
        params = {
            "id": video_id,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="DELETE",
            path="videos",
            params=params,
        )
        if response.ok:
            return True
        self._client.parse_response(response=response)



"""
    Comment threads resource implementation.
"""
from typing import Optional, Union

from pyyoutube.error import PyYouTubeException, ErrorMessage, ErrorCode
from pyyoutube.resources.base_resource import Resource
from pyyoutube.models import CommentThread, CommentThreadListResponse
from pyyoutube.utils.params_checker import enf_parts


class CommentThreadsResource(Resource):
    """A commentThread resource contains information about a YouTube comment thread, which comprises a
    top-level comment and replies, if any exist, to that comment

    References: https://developers.google.com/youtube/v3/docs/commentThreads
    """

    def list(
        self,
        parts: Optional[Union[str, list, tuple, set]] = None,
        all_threads_related_to_channel_id: Optional[str] = None,
        channel_id: Optional[str] = None,
        thread_id: Optional[Union[str, list, tuple, set]] = None,
        video_id: Optional[str] = None,
        max_results: Optional[int] = None,
        moderation_status: Optional[str] = None,
        order: Optional[str] = None,
        page_token: Optional[str] = None,
        search_terms: Optional[str] = None,
        text_format: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, CommentThreadListResponse]:
        """Returns a list of comment threads that match the API request parameters.

        Args:
            parts:
                Comma-separated list of one or more comment thread resource properties.
            all_threads_related_to_channel_id:
                Instructs the API to return all comment threads associated with the specified channel.
            channel_id:
                Instructs the API to return comment threads containing comments about the specified channel
            thread_id:
                Specifies a comma-separated list of comment thread IDs for the resources that should be retrieved.
            video_id:
                Instructs the API to return comment threads associated with the specified video ID.
            max_results:
                The parameter specifies the maximum number of items that should be returned
                the result set.
                Acceptable values are 1 to 100, inclusive. The default value is 20.
            moderation_status:
                Set this parameter to limit the returned comment threads to a particular moderation state.
                The default value is published.
                Note: This parameter is not supported for use in conjunction with the id parameter.
            order:
                Specifies the order in which the API response should list comment threads.
                Valid values are:
                    - time: Comment threads are ordered by time. This is the default behavior.
                    - relevance: Comment threads are ordered by relevance.
                Notes: This parameter is not supported for use in conjunction with the `id` parameter.
            page_token:
                 Identifies a specific page in the result set that should be returned.
                 Notes: This parameter is not supported for use in conjunction with the `id` parameter.
            search_terms:
                 Instructs the API to limit the API response to only contain comments that contain
                 the specified search terms.
                 Notes: This parameter is not supported for use in conjunction with the `id` parameter.
            text_format:
                Set this parameter's value to html or plainText to instruct the API to return the comments
                left by users in html formatted or in plain text. The default value is html.
                Acceptable values are:
                    – html: Returns the comments in HTML format. This is the default value.
                    – plainText: Returns the comments in plain text format.
                Notes: This parameter is not supported for use in conjunction with the `id` parameter.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Comment threads data.
        Raises:
            PyYouTubeException: Missing filter parameter.
        """

        params = {
            "part": enf_parts(resource="commentThreads", value=parts),
            "maxResults": max_results,
            "moderationStatus": moderation_status,
            "order": order,
            "pageToken": page_token,
            "searchTerms": search_terms,
            "textFormat": text_format,
            **kwargs,
        }
        if all_threads_related_to_channel_id is not None:
            params["allThreadsRelatedToChannelId"] = all_threads_related_to_channel_id
        elif channel_id:
            params["channelId"] = channel_id
        elif thread_id:
            params["id"] = thread_id
        elif video_id:
            params["videoId"] = video_id
        else:
            raise PyYouTubeException(
                ErrorMessage(
                    status_code=ErrorCode.MISSING_PARAMS,
                    message="Specify at least one of all_threads_related_to_channel_id, channel_id, thread_id or video_id",
                )
            )
        response = self._client.request(path="commentThreads", params=params)
        data = self._client.parse_response(response=response)
        return data if return_json else CommentThreadListResponse.from_dict(data)

    def insert(
        self,
        body: Union[dict, CommentThread],
        parts: Optional[Union[str, list, tuple, set]] = None,
        return_json: bool = False,
        **kwargs,
    ) -> Union[dict, CommentThread]:
        """Creates a new top-level comment.

        Notes: To add a reply to an existing comment, use the comments.insert method instead.

        Args:
            body:
                Provide a commentThread resource in the request body. You can give dataclass or just a dict with data.
            parts:
                Comma-separated list of one or more comment thread resource properties.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Channel thread data.

        """
        params = {
            "part": enf_parts(resource="commentThreads", value=parts),
            **kwargs,
        }

        response = self._client.request(
            method="POST",
            path="commentThreads",
            params=params,
            json=body,
        )
        data = self._client.parse_response(response=response)
        return data if return_json else CommentThread.from_dict(data)



import re

try:
    import urllib.error as compat_urllib_error
except ImportError:  # Python 2
    import urllib2 as compat_urllib_error


# Pythons disagree on the type of a pattern (RegexObject, _sre.SRE_Pattern, Pattern, ...?)
COMPATIBLE_RE_PATTERN = type(re.compile(''))
# and on the type of a match
COMPATIBLE_RE_MATCH = type(re.match('a', 'a'))


__all__ = [
    compat_urllib_error,
    COMPATIBLE_RE_PATTERN,
    COMPATIBLE_RE_MATCH
]


"""
    These are comment threads related models.
"""
from dataclasses import dataclass, field
from typing import Optional, List

from .base import BaseModel
from .common import BaseResource, BaseApiResponse, PageInfo
from .comment import Comment


@dataclass
class CommentThreadSnippet(BaseModel):
    """A class representing comment tread snippet info.

    References: https://developers.google.com/youtube/v3/docs/commentThreads#snippet
    """

    channelId: Optional[str] = field(default=None)
    videoId: Optional[str] = field(default=None)
    topLevelComment: Optional[Comment] = field(default=None, repr=False)
    canReply: Optional[bool] = field(default=None, repr=False)
    totalReplyCount: Optional[int] = field(default=None, repr=False)
    isPublic: Optional[bool] = field(default=None, repr=False)


@dataclass
class CommentThreadReplies(BaseModel):
    """
    A class representing comment tread replies info.

    Refer: https://developers.google.com/youtube/v3/docs/commentThreads#replies
    """

    comments: Optional[List[Comment]] = field(default=None, repr=False)
    # 공식 & 비공식 API 간에 요청 파라미터 차이로 인해, 아래 항목 추가함
    nextPageToken: Optional[str] = field(default=None, repr=False)
    prevPageToken: Optional[str] = field(default=None, repr=False)
    pageInfo: Optional[PageInfo] = field(default=None, repr=False)

@dataclass
class CommentThread(BaseResource):
    """
    A class representing comment thread info.

    Refer: https://developers.google.com/youtube/v3/docs/commentThreads
    """

    snippet: Optional[CommentThreadSnippet] = field(default=None, repr=False)
    replies: Optional[CommentThreadReplies] = field(default=None, repr=False)


@dataclass
class CommentThreadListResponse(BaseApiResponse):
    """
    A class representing the comment thread's retrieve response info.

    Refer: https://developers.google.com/youtube/v3/docs/commentThreads/list#response_1
    """

    items: Optional[List[CommentThread]] = field(default=None, repr=False)



import requests
from typing import Union, Optional

from ytvs.client.unofficial.resources.base_resource import Resource
from ytvs.extractor import YoutubeBasicInfoExtractor
from ytvs.utils import try_get, str_to_int, number_str_abbr_to_int, is_korean_number


class ChannelResource(Resource):
    """
    YouTube 채널과 관련된 정보를 제공하는 클래스.
    이 클래스는 Resource 클래스를 상속받아 YouTube 채널 데이터에 접근하고 파싱하는 기능을 제공.
    """
    KIND = 'youtube#channel'   # YouTube 채널의 종류를 나타내는 상수
    RESOURCE = 'channel'   # 자원의 이름을 나타내는 상수

    def get_channel_info(self,
                         channel_id: Optional[str] = None,
                         parts: Optional[str] = None):
        """
        YouTube 채널의 정보를 가져오는 함수
        :param channel_id: 정보를 가져올 채널 ID.
        :param parts: 요청할 데이터의 특정 부분을 나타내는 매개변수입니다.
        :return: 파싱된 채널 데이터를 반환합니다.
        """
        # 1. 영상 페이지 요청
        data = self._request_data(channel_id)

        # 2. 영상 페이지에서 데이터 추출
        yt_initial_data = YoutubeBasicInfoExtractor.extract_yt_initial_data(data)

        # 3. 데이터 분석 및 파싱
        parsed_data = self._parse_channel_data_from_initial_data(yt_initial_data)
        return parsed_data

    def _request_data(self,
                      channel_id: str):
        """
        YouTube 채널 페이지의 HTML 데이터를 요청하는 함수ㅌ.
        :param channel_id: 요청할 채널의 ID.
        :return: 채널 페이지의 HTML 텍스트를 반환.
        """
        # 1. Channel resource 요청 URL 생성
        endpoint = self.build_channel_resource_url(channel_id)

        # 2. Channel 페이지 요청
        html_text = self._client.fetch_url(endpoint) \
            .decode('utf-8')
        
        # 3. Channel 페이지 html 반환
        return html_text

    def _parse_channel_data_from_initial_data(self, initial_data):
        """
        초기 데이터에서 YouTube 채널 데이터를 파싱하는 함수.
        :param initial_data: YouTube 채널 페이지에서 추출된 초기 데이터.
        :return: 파싱된 채널 데이터를 반환.
       :return:
        """
        # 1. 데이터 분석 및 파싱
        header_raw = try_get(initial_data, lambda x: x['init_data']['header']['c4TabbedHeaderRenderer'])
        header = YoutubeBasicInfoExtractor.extract_channel_header(header_raw)
        metadata_raw = try_get(initial_data, lambda x: x['init_data']['metadata']['channelMetadataRenderer'])
        metadata = YoutubeBasicInfoExtractor.extract_metadata(metadata_raw)
        topbar_raw = try_get(initial_data, lambda x: x['init_data']['topbar']['desktopTopbarRenderer'])
        topbar = YoutubeBasicInfoExtractor.extract_channel_topbar(topbar_raw)

        return {
            'etag': None,
            'kind': 'youtube#channelListResponse',
            'items': [{
                'id': header['channel_id'],
                'kind': self.KIND,
                'localizations': None,
                'etag': None,
                'snippet': {
                    'title': metadata['title'],
                    'description': metadata['description'],
                    'country': topbar['country_code'],
                    'customUrl': header['channel_handle_text'],
                    'publishedAt': None,
                    'thumbnails': header['banner_urls'],
                },
                'statistics': {
                    'subscriberCount': header['subscriber_count'],
                    'videoCount': header['video_count'],
                    'viewCount': None,
                }
            }],
            'nextPageToken': None,
            'prevPageToken': None,
            'pageInfo': {
                'totalResults': 1,
                'resultsPerPage': 5
            }
        }



from typing import Optional, TYPE_CHECKING
# TYPE_CHECKING: 런타임 시에는 무시되고, 타입 체크 도구에서만 사용
if TYPE_CHECKING:
    # 비공식 YouTube 클라이언트를 임포트합니다. 이는 타입 체킹을 위해서만 사용
    from ytvs.client.unofficial.client import UnofficialClient


class Resource:
    """ Resource base class
    YouTube 비공식 API에 접근하기 위한 기본 자원 클래스
    """

    # YouTube 데스크톱 및 모바일 버전의 기본 URL
    DESKTOP_URL = f'https://www.youtube.com/'
    MOBILE_URL = f'https://www.youtube.com/'

    # YouTube API의 특정 기능에 접근하기 위한 URL들
    SEARCH_V1_URL = 'https://m.youtube.com/youtubei/v1/search'
    CONTINUATION_URL = 'https://m.youtube.com/youtubei/v1/next'
    TRANSCRIPT_API = 'https://m.youtube.com/youtubei/v1/get_transcript'

    # YouTube API에 접근하기 위한 키
    API_KEY = 'AIzaSyAO_FJ2SlqU8Q4STEHLGCilw_Y9_11qcW8'

    # YouTube API 요청 시 사용되는 클라이언트 정보
    CONTEXT_PARAM = {
        'context': {
            'client': {
                'hl': 'ko',  # en
                'gl': 'KR',  # US
                'clientName': 'WEB',  # MWEB
                'clientVersion': '2.20210804.02.00',
            },
        }
    }

    def __init__(self, client: Optional["UnofficialClient"] = None):
        """
        # Resource 클래스의 생성자입니다. 비공식 클라이언트 인스턴스를 선택적으로 받습니다.
        :param client:
        """
        self._client = client

    def build_video_resource_url(self, video_id:str):
        """
        YouTube 비디오의 URL을 생성합니다.
        :param video_id: 생성할 URL의 비디오 ID입니다.
        :return: YouTube 비디오의 전체 URL을 반환합니다.
        """
        return f'{self.DESKTOP_URL}watch?v={video_id}&hl=ko'

    def build_transcript_resource_url(self):
        """
        YouTube 동영상의 자막 정보에 접근하기 위한 API URL을 생성합니다.
        :return: 자막 정보에 접근하기 위한 API URL을 반환합니다.
        """
        return f'{self.TRANSCRIPT_API}?key={self.API_KEY}'

    def build_channel_resource_url(self, channel_id:str):
        """
        특정 YouTube 채널의 URL을 생성합니다.
        :param channel_id: 생성할 URL의 채널 ID입니다.
        :return: YouTube 채널의 전체 URL을 반환합니다.
        """
        return f'{self.DESKTOP_URL}channel/{channel_id}'

    def build_continuous_resource_url(self):
        """
        YouTube API에서 연속된 데이터에 접근하기 위한 URL을 생성합니다.
        :return: 연속된 데이터에 접근하기 위한 API URL을 반환합니다.
        """
        return f'{self.CONTINUATION_URL}/?key={self.API_KEY}'

    def build_search_resource_url(self, keyword_query, filter_query):
        """
        YouTube 검색 결과에 접근하기 위한 URL을 생성합니다.
        :param keyword_query: 검색어입니다.
        :param filter_query: 검색 결과를 필터링하는 쿼리입니다.
        :return: 검색 결과에 접근하기 위한 URL을 반환합니다.
        """
        return f'{self.DESKTOP_URL}/results?search_query={keyword_query}&sp={filter_query}'

    def build_search_v1_resource_url(self):
        """
        YouTube의 새로운 검색 API에 접근하기 위한 URL을 생성합니다.
        :param c_token: 검색 API에 접근하기 위한 토큰입니다.
        :return: 검색 API에 접근하기 위한 URL을 반환합니다.
        """
        return f'{self.SEARCH_V1_URL}?key={self.API_KEY}'



import urllib3


class ProxyManagerPool:
    def __init__(self, proxies):
        """
        proxies: 스키마를 키로 하고 프록시 URL 목록을 값으로 하는 딕셔너리
                 예: {'http': ['http://proxy1.com', 'http://proxy2.com'], 'https': ['https://proxy1.com']}
        """
        self.proxies = proxies
        self.managers = {}
        for schema, proxy in self.proxies.items():
            self.managers[schema] = urllib3.ProxyManager(proxy_url=proxy)
            #self.managers[schema] = [urllib3.ProxyManager(proxy_url=proxy) for proxy in proxy_list]

    def request(self, method, url, **kwargs):
        """
        주어진 URL의 스키마에 맞는 프록시 매니저를 사용하여 요청을 보냅니다.
        """
        schema = url.split(':')[0]  # URL에서 스키마 추출 (예: 'http', 'https')
        if schema in self.managers:
            # 스키마에 해당하는 첫 번째 프록시 매니저 사용
            # 필요에 따라 로드 밸런싱이나 장애 대비를 위한 로직 추가 가능
            manager = self.managers[schema]
            return manager.request(method, url, **kwargs)
        else:
            raise ValueError("Unsupported schema: {}".format(schema))


"""
    These are search result related models.
"""
from dataclasses import dataclass, field
from typing import Optional, List

from .base import BaseModel
from .common import BaseApiResponse, BaseResource, Thumbnails
from .mixins import DatetimeTimeMixin


@dataclass
class SearchResultSnippet(BaseModel, DatetimeTimeMixin):
    """
    A class representing the search result snippet info.

    Refer: https://developers.google.com/youtube/v3/docs/search#snippet
    """

    publishedAt: Optional[str] = field(default=None, repr=False)
    channelId: Optional[str] = field(default=None)
    title: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None, repr=False)
    thumbnails: Optional[Thumbnails] = field(default=None, repr=False)
    channelTitle: Optional[str] = field(default=None, repr=False)
    liveBroadcastContent: Optional[str] = field(default=None, repr=False)
    duration: Optional[str] = field(default=None, repr=False)


@dataclass
class SearchResultId(BaseModel):
    """
    A class representing the search result id info.

    Refer: https://developers.google.com/youtube/v3/docs/search#id
    """

    kind: Optional[str] = field(default=None)
    videoId: Optional[str] = field(default=None, repr=False)
    channelId: Optional[str] = field(default=None, repr=False)
    playlistId: Optional[str] = field(default=None, repr=False)


@dataclass
class SearchResult(BaseResource):
    """
    A class representing the search result's info.

    Refer: https://developers.google.com/youtube/v3/docs/search
    """

    id: Optional[SearchResultId] = field(default=None, repr=False)
    snippet: Optional[SearchResultSnippet] = field(default=None, repr=False)


@dataclass
class SearchListResponse(BaseApiResponse):
    """
    A class representing the channel's retrieve response info.

    Refer: https://developers.google.com/youtube/v3/docs/channels/list#response_1
    """

    regionCode: Optional[str] = field(default=None, repr=False)
    items: Optional[List[SearchResult]] = field(default=None, repr=False)



"""
    Playlist items resource implementation.
"""

from typing import Optional, Union

from pyyoutube.error import PyYouTubeException, ErrorCode, ErrorMessage
from pyyoutube.resources.base_resource import Resource
from pyyoutube.models import PlaylistItem, PlaylistItemListResponse
from pyyoutube.utils.params_checker import enf_parts, enf_comma_separated


class PlaylistItemsResource(Resource):
    """A playlistItem resource identifies another resource, such as a video, that is included
    in a playlist. In addition, the playlistItem resource contains details about the included
    resource that pertain specifically to how that resource is used in that playlist.

    References: https://developers.google.com/youtube/v3/docs/playlistItems
    """

    def list(
        self,
        parts: Optional[Union[str, list, tuple, set]] = None,
        playlist_item_id: Optional[Union[str, list, tuple, set]] = None,
        playlist_id: Optional[str] = None,
        max_results: Optional[int] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        page_token: Optional[str] = None,
        video_id: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, PlaylistItemListResponse]:
        """Returns a collection of playlist items that match the API request parameters.

        Args:
            parts:
                Comma-separated list of one or more channel resource properties.
                Accepted values: id,contentDetails,snippet,snippet
            playlist_item_id:
                Specifies a comma-separated list of one or more unique playlist item IDs.
            playlist_id:
                Specifies the unique ID of the playlist for which you want to retrieve playlist items.
            max_results:
                The parameter specifies the maximum number of items that should be returned
                the result set.
                Acceptable values are 0 to 50, inclusive. The default value is 5.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many difference YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            page_token:
                The parameter identifies a specific page in the result set that should be returned.
            video_id:
                Specifies that the request should return only the playlist items that contain the specified video.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Playlist items data.

        Raises:
            PyYouTubeException: Missing filter parameter.
        """

        params = {
            "part": enf_parts(resource="playlistItems", value=parts),
            "maxResults": max_results,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            "videoId": video_id,
            "pageToken": page_token,
            **kwargs,
        }
        if playlist_item_id is not None:
            params["id"] = enf_comma_separated(
                field="playlist_item_id", value=playlist_item_id
            )
        elif playlist_id is not None:
            params["playlistId"] = playlist_id
        else:
            raise PyYouTubeException(
                ErrorMessage(
                    status_code=ErrorCode.MISSING_PARAMS,
                    message=f"Specify at least one of playlist_item_id or playlist_id",
                )
            )

        response = self._client.request(path="playlistItems", params=params)
        data = self._client.parse_response(response=response)
        return data if return_json else PlaylistItemListResponse.from_dict(data)

    def insert(
        self,
        body: Union[dict, PlaylistItem],
        parts: Optional[Union[str, list, tuple, set]] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, PlaylistItem]:
        """Adds a resource to a playlist.

        Args:
            body:
                Provide playlist item data in the request body. You can give dataclass or just a dict with data.
            parts:
                Comma-separated list of one or more channel resource properties.
                Accepted values: id,contentDetails,snippet,snippet
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many difference YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Playlist item data.
        """
        params = {
            "part": enf_parts(resource="playlistItems", value=parts),
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="POST",
            path="playlistItems",
            params=params,
            json=body,
        )
        data = self._client.parse_response(response=response)
        return data if return_json else PlaylistItem.from_dict(data)

    def update(
        self,
        body: Union[dict, PlaylistItem],
        parts: Optional[Union[str, list, tuple, set]] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, PlaylistItem]:
        """Modifies a playlist item. For example, you could update the item's position in the playlist.

        Args:
            body:
                Provide playlist item data in the request body. You can give dataclass or just a dict with data.
            parts:
                Comma-separated list of one or more channel resource properties.
                Accepted values: id,contentDetails,snippet,snippet
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many difference YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Playlist item update data.
        """
        params = {
            "part": enf_parts(resource="playlistItems", value=parts),
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="PUT",
            path="playlistItems",
            params=params,
            json=body,
        )
        data = self._client.parse_response(response=response)
        return data if return_json else PlaylistItem.from_dict(data)

    def delete(
        self,
        playlist_item_id: str,
        on_behalf_of_content_owner: Optional[str] = None,
        **kwargs: Optional[dict],
    ) -> bool:
        """Deletes a playlist item.

        Args:
            playlist_item_id:
                Specifies the YouTube playlist item ID for the playlist item that is being deleted.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many difference YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Playlist item delete status.

        """
        params = {
            "id": playlist_item_id,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="DELETE",
            path="playlistItems",
            params=params,
        )
        if response.ok:
            return True
        self._client.parse_response(response=response)



from .transcript import TranscriptResource
from .channel import ChannelResource
from .comment_reply import CommentResource
from .comment_thread import CommentThreadResource
from .search  import SearchResource
from .video import VideoResource



__all__ = [
    TranscriptResource,
    ChannelResource,
    CommentResource,
    CommentThreadResource,
    SearchResource,
    VideoResource
]




"""
    Channel resource implementation.
"""
from typing import Optional, Union

from pyyoutube.error import PyYouTubeException, ErrorMessage, ErrorCode
from pyyoutube.resources.base_resource import Resource
from pyyoutube.models import Channel, ChannelListResponse
from pyyoutube.utils.params_checker import enf_comma_separated, enf_parts


class ChannelsResource(Resource):
    """A channel resource contains information about a YouTube channel.

    References: https://developers.google.com/youtube/v3/docs/channels
    """

    def list(
        self,
        parts: Optional[Union[str, list, tuple, set]] = None,
        for_username: Optional[str] = None,
        channel_id: Optional[Union[str, list, tuple, set]] = None,
        managed_by_me: Optional[bool] = None,
        mine: Optional[bool] = None,
        hl: Optional[str] = None,
        max_results: Optional[int] = None,
        on_behalf_of_content_owner: Optional[str] = None,
        page_token: Optional[str] = None,
        return_json: bool = False,
        **kwargs: Optional[dict],
    ) -> Union[dict, ChannelListResponse]:
        """Returns a collection of zero or more channel resources that match the request criteria.

        Args:
            parts:
                Comma-separated list of one or more channel resource properties.
                Accepted values: id,auditDetails,brandingSettings,contentDetails,contentOwnerDetails,
                localizations,snippet,statistics,status,topicDetails
            for_username:
                The parameter specifies a YouTube username, thereby requesting
                the channel associated with that username.
            channel_id:
                The parameter specifies a comma-separated list of the YouTube channel ID(s)
                for the resource(s) that are being retrieved.
            managed_by_me:
                Set this parameter's value to true to instruct the API to only return channels
                managed by the content owner that the onBehalfOfContentOwner parameter specifies.
                The user must be authenticated as a CMS account linked to the specified content
                owner and onBehalfOfContentOwner must be provided.
            mine:
                Set this parameter's value to true to instruct the API to only return channels
                owned by the authenticated user.
            hl:
                The hl parameter instructs the API to retrieve localized resource metadata for
                a specific application language that the YouTube website supports.
                The parameter value must be a language code included in the list returned by the
                i18nLanguages.list method.
            max_results:
                The parameter specifies the maximum number of items that should be returned
                the result set.
                Acceptable values are 0 to 50, inclusive. The default value is 5.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many difference YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            page_token:
                The parameter identifies a specific page in the result set that should be returned.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Channel data
        Raises:
            PyYouTubeException: Missing filter parameter.
                                Request not success.
        """

        params = {
            "part": enf_parts(resource="channels", value=parts),
            "hl": hl,
            "maxResults": max_results,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            "pageToken": page_token,
            **kwargs,
        }
        if for_username is not None:
            params["forUsername"] = for_username
        elif channel_id is not None:
            params["id"] = enf_comma_separated(field="channel_id", value=channel_id)
        elif managed_by_me is not None:
            params["managedByMe"] = managed_by_me
        elif mine is not None:
            params["mine"] = mine
        else:
            raise PyYouTubeException(
                ErrorMessage(
                    status_code=ErrorCode.MISSING_PARAMS,
                    message=f"Specify at least one of for_username,channel_id,managedByMe or mine",
                )
            )

        response = self._client.request(path="channels", params=params)
        data = self._client.parse_response(response=response)
        return data if return_json else ChannelListResponse.from_dict(data)

    def update(
        self,
        part: str,
        body: Union[dict, Channel],
        on_behalf_of_content_owner: Optional[str] = None,
        return_json: bool = False,
        **kwargs,
    ) -> Union[dict, Channel]:
        """Updates a channel's metadata.

        Note that this method currently only supports updates to the channel resource's brandingSettings,
        invideoPromotion, and localizations objects and their child properties.

        Args:
            part:
                The part parameter serves two purposes in this operation. It identifies the properties
                that the write operation will set as well as the properties that the API response will include.
            body:
                Provide channel data in the request body. You can give dataclass or just a dict with data.
            on_behalf_of_content_owner:
                The onBehalfOfContentOwner parameter indicates that the request's authorization
                credentials identify a YouTube CMS user who is acting on behalf of the content
                owner specified in the parameter value. This parameter is intended for YouTube
                content partners that own and manage many different YouTube channels. It allows
                content owners to authenticate once and get access to all their video and channel
                data, without having to provide authentication credentials for each individual channel.
                The CMS account that the user authenticates with must be linked to the specified YouTube content owner.
            return_json:
                Type for returned data. If you set True JSON data will be returned.
            **kwargs:
                Additional parameters for system parameters.
                Refer: https://cloud.google.com/apis/docs/system-parameters.

        Returns:
            Channel updated data.
        """

        params = {
            "part": part,
            "onBehalfOfContentOwner": on_behalf_of_content_owner,
            **kwargs,
        }
        response = self._client.request(
            method="PUT",
            path="channels",
            params=params,
            json=body,
        )
        data = self._client.parse_response(response=response)
        return data if return_json else Channel.from_dict(data)
